"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("underscore");
const utils_1 = require("@nodeswork/utils");
var filters;
(function (filters) {
    function parse() {
        let items = _.flatten(arguments);
        for (let i = 0; i < items.length; i++) {
            if (_.isString(items[i])) {
                let descriptor = items[i];
                let required = false;
                items[i] = [];
                if (descriptor.startsWith('$')) {
                    required = true;
                    descriptor = descriptor.substring(1);
                }
                let [path, ...filterNames] = descriptor.split(':');
                items[i].push(extract(path));
                if (required) {
                    items[i].push(Required);
                }
                for (let name of filterNames) {
                    let f = filters[name];
                    if (f == null) {
                        throw new utils_1.NodesworkError('Filter is missing', {
                            filter: name,
                            descriptor: descriptor,
                        });
                    }
                    items[i].push(f);
                }
            }
            else if (_.isFunction(items[i])) {
                continue;
            }
            else {
                throw new utils_1.NodesworkError('Unkown param filter', {
                    item: items[i],
                });
            }
        }
        items = _.flatten(items);
        return async function (val, ctx) {
            let paths = [];
            let ret = val;
            try {
                for (let f of items) {
                    let p = f.path;
                    if (p) {
                        paths.push(p);
                    }
                    ret = await f(ret, ctx);
                }
                return ret;
            }
            catch (e) {
                if (e instanceof utils_1.NodesworkError) {
                    e.meta.responseCode = 400;
                    e.meta.path = paths.join('.');
                    throw e;
                }
                else {
                    console.error(e);
                    throw new utils_1.NodesworkError('internal service error', {
                        responseCode: 500,
                    }, e);
                }
            }
        };
    }
    filters.parse = parse;
    function extract(path) {
        let paths = path.split('.');
        function Extract(val) {
            let ret = val;
            for (let p of paths) {
                if (ret == null) {
                    break;
                }
                ret = ret[p];
            }
            return ret;
        }
        ;
        Extract.path = path;
        return Extract;
    }
    filters.extract = extract;
    function Required(val) {
        if (val == null) {
            throw new utils_1.NodesworkError('Missing value');
        }
        return val;
    }
    filters.Required = Required;
    const trueValues = ['1', 'true', 1, true];
    const falseValues = ['0', 'false', 0, false];
    function Boolean(val) {
        if (val == null) {
            return null;
        }
        if (trueValues.indexOf(val) >= 0) {
            return true;
        }
        if (falseValues.indexOf(val) >= 0) {
            return false;
        }
        throw new utils_1.NodesworkError('Invalid boolean value', {
            value: val,
        });
    }
    filters.Boolean = Boolean;
    function Integer(val) {
        if (val == null) {
            return null;
        }
        let ret = parseInt(val);
        if (_.isNaN(ret)) {
            throw new utils_1.NodesworkError('Invalid integer value', {
                value: val,
            });
        }
        return ret;
    }
    filters.Integer = Integer;
    function Json(val) {
        if (val == null) {
            return null;
        }
        try {
            return JSON.parse(val);
        }
        catch (e) {
            throw new utils_1.NodesworkError('Invalid json value', {
                value: val,
                error: e.message,
            });
        }
    }
    filters.Json = Json;
    function setDefault(defaultVal) {
        return function Default(val) {
            return val == null ? defaultVal : val;
        };
    }
    filters.setDefault = setDefault;
})(filters = exports.filters || (exports.filters = {}));

//# sourceMappingURL=params.js.map
