"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("underscore");
const utils_1 = require("@nodeswork/utils");
const model = require("./model");
exports.READONLY = 'READONLY';
exports.AUTOGEN = 'AUTOGEN';
class KoaMiddlewares extends model.Model {
    static createMiddleware(options) {
        const self = this.cast();
        _.defaults(options, DEFAULT_COMMON_OPTIONS);
        async function create(ctx, next) {
            let rModel = self;
            const omits = _.union(['_id'], options.omits, self.schema.api.AUTOGEN);
            let doc = _.omit(ctx.request.body, omits);
            doc = _.extend(doc, ctx.overrides && ctx.overrides.doc);
            const discriminatorKey = self.schema.options.discriminatorKey;
            if (discriminatorKey && discriminatorKey !== '__t') {
                const modelName = doc[discriminatorKey];
                if (modelName) {
                    try {
                        rModel = self.db.model(modelName);
                    }
                    catch (e) {
                        /* handle error */
                        rModel = null;
                    }
                }
                if (!options.allowCreateFromParentModel && !modelName) {
                    throw new utils_1.NodesworkError('required field is missing', {
                        responseCode: 422,
                        path: discriminatorKey,
                    });
                }
                if (!rModel) {
                    throw new utils_1.NodesworkError('invalid value', {
                        responseCode: 422,
                        path: discriminatorKey,
                    });
                }
            }
            ctx[options.target] = doc;
            if (options.triggerNext) {
                await next();
            }
            doc = ctx[options.target];
            let object = (await rModel.create(doc));
            if (options.project || options.level) {
                object = await self.findById(object._id, options.project, {
                    level: options.level,
                });
            }
            ctx[options.target] = object;
            if (options.populate) {
                await rModel.populate(object, options.populate);
            }
            if (!options.noBody) {
                ctx.body = await options.transform(object);
            }
        }
        Object.defineProperty(create, 'name', {
            value: `${self.modelName}#createMiddleware`,
            writable: false,
        });
        return create;
    }
    static getMiddleware(options) {
        const self = this.cast();
        _.defaults(options, DEFAULT_COMMON_OPTIONS);
        async function get(ctx, next) {
            const query = ctx.overrides && ctx.overrides.query || {};
            if (options.field !== '*') {
                query._id = ctx.params[options.field];
                if (query._id == null) {
                    throw new utils_1.NodesworkError('invalid value', {
                        responseCode: 422,
                        path: options.field,
                    });
                }
            }
            if (Object.keys(query).length === 0) {
                throw new utils_1.NodesworkError('no query parameters', {
                    responseCode: 422,
                    path: options.field,
                });
            }
            const queryOption = {};
            if (options.level) {
                queryOption.level = options.level;
            }
            let queryPromise = self.findOne(query, options.project, queryOption);
            if (options.populate) {
                queryPromise = queryPromise.populate(options.populate);
            }
            const object = await queryPromise;
            ctx[options.target] = object;
            if (!options.nullable && object == null) {
                throw utils_1.NodesworkError.notFound();
            }
            if (options.triggerNext) {
                await next();
            }
            if (!options.noBody) {
                ctx.body = await options.transform(ctx[options.target]);
            }
        }
        Object.defineProperty(get, 'name', {
            value: `${self.modelName}#getMiddleware`,
            writable: false,
        });
        return get;
    }
    static findMiddleware(options) {
        const self = this.cast();
        _.defaults(options, DEFAULT_COMMON_OPTIONS);
        if (options.pagination) {
            _.defaults(options.pagination, DEFAULT_FIND_PAGINATION_OPTIONS);
        }
        const defaultPagination = {
            page: 0,
            size: options.pagination ? options.pagination.size : 0,
        };
        async function find(ctx, next) {
            const query = ctx.overrides && ctx.overrides.query || {};
            const queryOption = {};
            let pagination = null;
            if (options.pagination) {
                // pagination       = VALIDATE_QUERY_PAGINATION(ctx.request.query);
                pagination = ctx.request.query;
                _.defaults(pagination, defaultPagination);
                queryOption.skip = pagination.page * pagination.size;
                queryOption.limit = pagination.size;
            }
            if (options.level) {
                queryOption.level = options.level;
            }
            let queryPromise = self.find(query, options.project, queryOption);
            if (options.populate) {
                queryPromise = queryPromise.populate(options.populate);
            }
            const object = await queryPromise;
            ctx[options.target] = object;
            if (options.triggerNext) {
                await next();
            }
            if (pagination) {
                const totalPage = Math.floor((await self.find(query).count() - 1) / pagination.size + 1);
                ctx.response.set('total_page', totalPage.toString());
            }
            if (!options.noBody) {
                const body = ctx[options.target];
                for (let i = 0; i < body.length; i++) {
                    body[i] = await options.transform(body[i]);
                }
                ctx.body = body;
            }
        }
        Object.defineProperty(find, 'name', {
            value: `${self.modelName}#findMiddleware`,
            writable: false,
        });
        return find;
    }
    static updateMiddleware(options) {
        const self = this.cast();
        _.defaults(options, DEFAULT_COMMON_OPTIONS);
        async function update(ctx, next) {
            const query = ctx.overrides && ctx.overrides.query || {};
            if (options.field !== '*') {
                query._id = ctx.params[options.field];
                if (query._id == null) {
                    throw new utils_1.NodesworkError('invalid value', {
                        responseCode: 422,
                        path: options.field,
                    });
                }
            }
            if (Object.keys(query).length === 0) {
                throw new utils_1.NodesworkError('no query parameters', {
                    responseCode: 422,
                    path: options.field,
                });
            }
            const queryOption = {
                new: true,
                fields: options.project,
                level: options.level,
                runValidators: true,
            };
            const omits = _.union(['_id'], options.omits, self.schema.api.READONLY, self.schema.api.AUTOGEN);
            let doc = _.omit(ctx.request.body, omits);
            doc = _.extend(doc, ctx.overrides && ctx.overrides.doc);
            const upDoc = {
                $set: doc,
            };
            let rModel = self;
            const discriminatorKey = self.schema.options.discriminatorKey;
            const modelName = ctx.request.body[discriminatorKey];
            if (discriminatorKey && modelName) {
                try {
                    rModel = self.db.model(modelName);
                    query[discriminatorKey] = modelName;
                }
                catch (e) {
                    /* handle error */
                }
            }
            let updatePromise = rModel.findOneAndUpdate(query, upDoc, queryOption);
            if (options.populate) {
                updatePromise = updatePromise.populate(options.populate);
            }
            const object = await updatePromise;
            ctx[options.target] = object;
            if (options.triggerNext) {
                await next();
            }
            if (!options.noBody) {
                ctx.body = await options.transform(ctx[options.target]);
            }
        }
        Object.defineProperty(update, 'name', {
            value: `${self.modelName}#updateMiddleware`,
            writable: false,
        });
        return update;
    }
    static deleteMiddleware(options) {
        const self = this.cast();
        _.defaults(options, DEFAULT_COMMON_OPTIONS);
        async function del(ctx, next) {
            const query = ctx.overrides && ctx.overrides.query || {};
            query._id = ctx.params[options.field];
            const queryOption = {};
            const queryPromise = self.findOne(query, undefined, queryOption);
            let object = await queryPromise;
            ctx[options.target] = object;
            if (options.triggerNext) {
                await next();
            }
            object = ctx[options.target];
            if (!options.nullable && object == null) {
                throw new utils_1.NodesworkError('not found', {
                    responseCode: 404,
                });
            }
            if (object) {
                await object.delete();
            }
            ctx.status = 204;
        }
        Object.defineProperty(del, 'name', {
            value: `${self.modelName}#deleteMiddleware`,
            writable: false,
        });
        return del;
    }
}
exports.KoaMiddlewares = KoaMiddlewares;
const DEFAULT_COMMON_OPTIONS = {
    target: 'object',
    transform: _.identity,
};
const DEFAULT_FIND_PAGINATION_OPTIONS = {
    size: 20,
    sizeChoices: [20, 50, 100, 200],
};
const VALIDATE_QUERY_PAGINATION = utils_1.validator2.compile({
    page: [],
    size: [],
});
KoaMiddlewares.Plugin({
    fn: apiLevel,
});
function apiLevel(schema, options) {
    if (schema.api == null) {
        schema.api = {
            READONLY: [],
            AUTOGEN: [],
        };
    }
    schema.eachPath((pathname, schemaType) => {
        if ([exports.READONLY, exports.AUTOGEN].indexOf(schemaType.options.api) < 0) {
            return;
        }
        for (let s = schema; s != null; s = s.parentSchema) {
            s.api[schemaType.options.api] = _.union(s.api[schemaType.options.api], [pathname]);
        }
    });
}

//# sourceMappingURL=koa.js.map
